package com.trafficparrot.sdk.file;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.github.tomakehurst.wiremock.matching.StringValuePattern;
import com.google.common.collect.ComparisonChain;
import com.trafficparrot.sdk.jms.Destination;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import static com.trafficparrot.sdk.jms.NamedValueMatcherComparator.NAMED_VALUE_MATCHER_COMPARATOR;
import static org.apache.commons.lang3.builder.ToStringBuilder.reflectionToString;
import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class FileMessageRequestMatcher implements Comparable<FileMessageRequestMatcher> {
    public final Destination destination;
    public final StringValuePattern fileNameMatcher;
    public final StringValuePattern bodyMatcher;

    public FileMessageRequestMatcher(@JsonProperty("destination") Destination destination,
                                     @JsonProperty("fileNameMatcher") StringValuePattern fileNameMatcher,
                                     @JsonProperty("bodyMatcher") StringValuePattern bodyMatcher) {
        this.destination = destination;
        this.fileNameMatcher = fileNameMatcher;
        this.bodyMatcher = bodyMatcher;
    }

    @Override
    @JsonIgnore
    public String toString() {
        return reflectionToString(this, SHORT_PREFIX_STYLE);
    }

    @Override
    public int hashCode() {
        return HashCodeBuilder.reflectionHashCode(this);
    }

    @Override
    public boolean equals(Object obj) {
        return EqualsBuilder.reflectionEquals(this, obj);
    }

    @Override
    public int compareTo(FileMessageRequestMatcher other) {
        return ComparisonChain.start()
                .compare(destination, other.destination)
                .compare(fileNameMatcher, other.fileNameMatcher, NAMED_VALUE_MATCHER_COMPARATOR)
                .compare(bodyMatcher, other.bodyMatcher, NAMED_VALUE_MATCHER_COMPARATOR)
                .result();
    }
}
