package com.trafficparrot.sdk.handlebars;

import com.github.jknack.handlebars.Options;

import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static java.lang.String.format;
import static java.util.regex.Pattern.DOTALL;
import static org.apache.commons.lang3.StringUtils.isBlank;

public class Regex extends TrafficParrotHelper<String> {

    @Override
    protected Object doApply(String inputText, Options options) throws IOException {
        if (isBlank(inputText)) {
            return "{{ Warning: input text parameter is missing }}";
        }

        if (options.param(0, null) == null) {
            return "{{ Warning: regex parameter is missing }}";
        }

        String inputRegex = options.param(0);
        int groups = numberOfGroups(inputRegex);
        if (groups == 0) {
            return "{{ Warning: regex parameter is missing a capturing group e.g. (.*) }}";
        }
        if (groups > 1) {
            return "{{ Warning: regex parameter must have only one capturing group }}";
        }
        if (inputRegex.equals("(.*)")) {
            return inputText;
        }
        for (String line : inputText.split("[\r\n]+")) {
            Pattern pattern = Pattern.compile(inputRegex);
            Matcher matcher = pattern.matcher(line);
            if (matcher.matches()) {
                String match = matcher.group(1);
                if (!match.isEmpty()) {
                    return match;
                }
            }
        }
        Pattern pattern = Pattern.compile(inputRegex, DOTALL);
        Matcher matcher = pattern.matcher(inputText);
        if (matcher.find()) {
            return matcher.group(1).trim();
        } else {
            return format("{{ Warning: regex '%s' did not match anything }}", inputRegex);
        }
    }

    private int numberOfGroups(String inputRegex) {
        return Pattern.compile(inputRegex).matcher("").groupCount();
    }
}
