package com.trafficparrot.sdk.http;

import com.github.tomakehurst.wiremock.common.FileSource;
import com.github.tomakehurst.wiremock.extension.Parameters;
import com.github.tomakehurst.wiremock.extension.ResponseDefinitionTransformer;
import com.github.tomakehurst.wiremock.http.Request;
import com.github.tomakehurst.wiremock.http.Response;
import com.github.tomakehurst.wiremock.http.ResponseDefinition;
import com.trafficparrot.sdk.jms.Slf4jTpLogger;
import com.trafficparrot.sdk.jms.TpLogger;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static java.lang.String.format;
import static javax.servlet.http.HttpServletResponse.SC_INTERNAL_SERVER_ERROR;

public abstract class HttpResponseDefinitionTransformer extends ResponseDefinitionTransformer {
    protected final TpLogger logger;

    protected HttpResponseDefinitionTransformer() {
        this.logger = new Slf4jTpLogger(HttpResponseDefinitionTransformer.class);
    }

    @Override
    public ResponseDefinition transform(Request request, ResponseDefinition responseDefinition, FileSource fileSource, Parameters parameters) {
        try {
            return doTransform(request, responseDefinition, fileSource, parameters);
        } catch (Exception e) {
            String errorMessage = format("%s had a problem while transforming: %s. See logs for more details.", getClass().getCanonicalName(), e.getMessage());
            logger.error(errorMessage, e);
            return new ResponseDefinition(SC_INTERNAL_SERVER_ERROR, errorMessage);
        }
    }

    public abstract ResponseDefinition doTransform(Request request, ResponseDefinition responseDefinition, FileSource fileSource, Parameters parameters);
}
