package com.trafficparrot.sdk.ibmmq;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.ComparisonChain;
import com.trafficparrot.sdk.jms.Destination;
import com.trafficparrot.sdk.jms.DestinationType;

import static com.trafficparrot.sdk.jms.DestinationType.QUEUE;
import static com.trafficparrot.sdk.jms.DestinationType.TOPIC;
import static org.apache.commons.lang3.builder.ToStringBuilder.reflectionToString;
import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class IbmMqMessageMapping implements Comparable<IbmMqMessageMapping> {

    public final String mappingId;
    public final IbmMqRequestMatcher request;
    public final IbmMqResponse response;

    public IbmMqMessageMapping(@JsonProperty("mappingId") String mappingId,
                               @JsonProperty("request") IbmMqRequestMatcher request,
                               @JsonProperty("response") IbmMqResponse response) {
        this.mappingId = mappingId;
        this.request = request;
        this.response = response;
    }

//    @JsonIgnore
//    public IbmMqResponse transformedResponse(Message requestMessage, IbmMqHandlebarsResponseTransformer handlebars) {
//        IbmMqResponse afterResponseTransformer = responseTransformerTransform(requestMessage);
//        IbmMqResponse afterHandlebars = handlebars.transform(afterResponseTransformer);
//        return afterHandlebars;
//    }

//    private IbmMqResponse responseTransformerTransform(Message requestMessage) {
//        if (StringUtils.isNotBlank(response.ibmMqResponseTransformerClassName) && !IbmMqResponse.NO_TRANSFORMER.equals(response.ibmMqResponseTransformerClassName)) {
//            IbmMqResponseTransformer jmsResponseTransformer = parseExtension(response.ibmMqResponseTransformerClassName);
//            return jmsResponseTransformer.transform(request.destination, requestMessage, response);
//        } else {
//            return response;
//        }
//    }

//    @JsonIgnore
//    private static IbmMqResponseTransformer parseExtension(String extensionsPropertyValue) {
//        try {
//            return Class.forName(extensionsPropertyValue).asSubclass(IbmMqResponseTransformer.class).newInstance();
//        } catch (ClassNotFoundException e) {
//            throw new IllegalArgumentException(format("Extension '%s' was not found on the classpath", extensionsPropertyValue), e);
//        } catch (IllegalAccessException | InstantiationException e) {
//            throw new IllegalArgumentException(format("Extension '%s' must have a public zero argument constructor", extensionsPropertyValue), e);
//        }
//    }

    @Override
    @JsonIgnore
    public String toString() {
        return reflectionToString(this, SHORT_PREFIX_STYLE);
    }

    @Override
    public int compareTo(IbmMqMessageMapping other) {
        return ComparisonChain.start()
                .compare(mappingId, other.mappingId)
                .compare(request, other.request)
                .compare(response, other.response)
                .result();
    }

    public static Predicate<IbmMqMessageMapping> isQueue() {
        return new Predicate<IbmMqMessageMapping>() {
            @Override
            public boolean apply(IbmMqMessageMapping input) {
                return QUEUE.equals(input.request.destination.type);
            }
        };
    }

    public static Predicate<IbmMqMessageMapping> isTopic() {
        return new Predicate<IbmMqMessageMapping>() {
            @Override
            public boolean apply(IbmMqMessageMapping input) {
                return TOPIC.equals(input.request.destination.type);
            }
        };
    }

    public static Function<IbmMqMessageMapping, String> destinationName() {
        return new Function<IbmMqMessageMapping, String>() {
            @Override
            public String apply(IbmMqMessageMapping input) {
                return input.request.destination.name;
            }
        };
    }

    public static Function<IbmMqMessageMapping, Destination> destination() {
        return new Function<IbmMqMessageMapping, Destination>() {
            @Override
            public Destination apply(IbmMqMessageMapping input) {
                return input.request.destination;
            }
        };
    }

    public static Predicate<? super IbmMqMessageMapping> withDestination(final Destination destination) {
        return new Predicate<IbmMqMessageMapping>() {
            @Override
            public boolean apply(IbmMqMessageMapping input) {
                return input.request.destination.equals(destination);
            }
        };
    }

    public static Predicate<? super IbmMqMessageMapping> withDestinationType(final DestinationType type) {
        return new Predicate<IbmMqMessageMapping>() {
            @Override
            public boolean apply(IbmMqMessageMapping input) {
                return input.request.destination.type == type;
            }
        };
    }
}
