package com.trafficparrot.sdk.jms;

import com.fasterxml.jackson.annotation.JsonProperty;
import org.apache.commons.lang3.builder.ReflectionToStringBuilder;

import javax.jms.JMSException;
import javax.jms.Message;
import java.lang.reflect.InvocationTargetException;

import static org.apache.commons.lang3.builder.ToStringStyle.SHORT_PREFIX_STYLE;

public class JmsProperty {
    public final String name;
    public final String type; //java.lang.Boolean, java.lang.Byte, java.lang.Short, java.lang.Integer, java.lang.Long, java.lang.Float, java.lang.Double, java.lang.String
    public final Object value;

    public JmsProperty(
            @JsonProperty("name") String name,
            @JsonProperty("type") String type,
            @JsonProperty("value") Object value) {
        this.name = name;
        this.type = type;
        this.value = value;
    }

    public void setProperty(Message message) {
        try {
            Object propertyValue = Class.forName(type).getConstructor(String.class).newInstance(String.valueOf(value));
            message.setObjectProperty(name, propertyValue);
        } catch (NoSuchMethodException | ClassNotFoundException | IllegalAccessException | JMSException | InstantiationException | InvocationTargetException e) {
            throw new IllegalArgumentException(toString(), e);
        }
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, SHORT_PREFIX_STYLE);
    }
}